@@ -1,21 +1,25 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
 from datetime import time
 from dateutil.parser import parse
 
 from formencode import validators as fev
+import six
+from six.moves import map
 
 class Currency(fev.Number):
-    symbol=u'$'
-    format=u'$%.2f'
+    symbol='$'
+    format='$%.2f'
     store_factor=100
 
     def _to_python(self, value, state):
-        value = value.strip(u' \r\n\t' + self.symbol)
+        value = value.strip(' \r\n\t' + self.symbol)
         return int(fev.Number._to_python(self, value, state) * self.store_factor)
 
     def from_python(self, value, state):
-        if isinstance(value, basestring):
+        if isinstance(value, six.string_types):
             return value
-        elif isinstance(value, (int, float, long)):
+        elif isinstance(value, six.integer_types + (float,)):
             value = float(value) / self.store_factor
             return self.format  % value
 
@@ -33,7 +37,7 @@ class DateConverter(fev.DateConverter):
         try:
             return parse(value).date()
         except ValueError:
-            return super(TimeConverter, self).to_python(value, state)
+            return super(DateConverter, self).to_python(value, state)
 
 class OneOf(fev.FancyValidator):
 
@@ -65,6 +69,15 @@ class OneOf(fev.FancyValidator):
 class UnicodeString(fev.UnicodeString):
 
     def from_python(self, value, state):
-        str_version = fev.UnicodeString.from_python(self, value, state)
-        return unicode(str_version, self.outputEncoding)
+        # before py3 conversions, if value was None, this function would raise an exception
+        #       TypeError: decoding Unicode is not supported
+        # and FieldWidget.prepare_context trapped all errors, so it wouldn't change its value
+        # so we preserve that resulting behavior (None stays None)
+        if value is None:
+            return None
+        str_version = super(UnicodeString, self).from_python(value, state)
+        return six.ensure_text(str_version, self.outputEncoding or 'utf-8')
 
+class Int(fev.Int):
+    def from_python(self, value, state):
+        return str(value)
